@@ -93,20 +93,11 @@ hello_application_window_init (HelloApplicationWindow *window)
   update_content (window);
 }
 
-static void
-hello_application_window_dispose (GObject *object)
-{
-  HelloApplicationWindow *window = HELLO_APPLICATION_WINDOW (object);
-  g_clear_object (&window->settings);
-}
-
 static void
 hello_application_window_class_init (HelloApplicationWindowClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  gobject_class->dispose = hello_application_window_dispose;
-
   gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass),
                                                UI_PATH);
   gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass),
@@ -179,8 +170,12 @@ main (int argc, char *argv[])
   GApplication *application;
   int status;
 
-  /* Load the GSettings schema from the current directory.  */
-  g_setenv ("GSETTINGS_SCHEMA_DIR", ".", FALSE);
+  /* Load the compiled GSettings schema
+     - from PKGDATADIR (so that it works after "make install"),
+     - from the current directory (so that it works in the build directory,
+       before "make install").  */
+  g_setenv ("GSETTINGS_SCHEMA_DIR", PKGDATADIR G_SEARCHPATH_SEPARATOR_S ".",
+            FALSE);
 
   /* Initializations.  */
   textdomain ("hello-c-gnome3");
