@@ -1,6 +1,6 @@
 /* Analyze differences between two vectors.
 
-   Copyright (C) 1988-1989, 1992-1995, 2001-2004, 2006-2023 Free Software
+   Copyright (C) 1988-1989, 1992-1995, 2001-2004, 2006-2024 Free Software
    Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -93,10 +93,14 @@
 #endif
 
 /* Suppress gcc's "...may be used before initialized" warnings,
-   generated by GCC versions up to at least GCC 13.2.  */
-#if __GNUC__ + (__GNUC_MINOR__ >= 7) > 4
+   generated by GCC versions up to at least GCC 14.2.
+   Likewise for gcc -fanalyzer's "use of uninitialized value" warnings.  */
+#if _GL_GNUC_PREREQ (4, 7)
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
+# if _GL_GNUC_PREREQ (13, 0)
+#  pragma GCC diagnostic ignored "-Wanalyzer-use-of-uninitialized-value"
+# endif
 #endif
 
 /*
@@ -554,7 +558,7 @@ compareseq (OFFSET xoff, OFFSET xlim, OFFSET yoff, OFFSET ylim,
   #undef XREF_YREF_EQUAL
 }
 
-#if __GNUC__ + (__GNUC_MINOR__ >= 7) > 4
+#if _GL_GNUC_PREREQ (4, 7)
 # pragma GCC diagnostic pop
 #endif
 
