@@ -1,10 +1,10 @@
 /* Waiting for a subprocess to finish.
-   Copyright (C) 2001-2003, 2005-2022 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003, 2005-2024 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -29,12 +29,12 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
-#include "error.h"
+#include <error.h>
 #include "fatal-signal.h"
 #include "xalloc.h"
 #include "gettext.h"
 
-#define _(str) gettext (str)
+#define _(msgid) dgettext ("gnulib", msgid)
 
 #define SIZEOF(a) (sizeof(a) / sizeof(a[0]))
 
@@ -50,6 +50,41 @@
 
 #endif
 
+#ifdef __KLIBC__
+# include <dlfcn.h>
+
+# undef waitpid
+
+/* Replacement of waitpid() to support spawn2() of LIBCx which is the kLIBC
+   extension library. See for details:
+   <https://github.com/bitwiseworks/libcx/blob/master/src/spawn/libcx/spawn2.h#L194>.
+   */
+static pid_t
+klibc_waitpid (pid_t pid, int *statusp, int options)
+{
+  static pid_t (*waitpid_pfn) (pid_t, int *, int) = NULL;
+
+  if (waitpid_pfn == NULL)
+    {
+      void *libcx_handle;
+
+      /* Try to use waitpid() of LIBCx first if available because it can
+         process the return value of spawn-family of kLIBC as well as spawn2()
+         of LIBCx.  */
+      libcx_handle = dlopen ("libcx0", RTLD_LAZY);
+      if (libcx_handle != NULL)
+        waitpid_pfn = dlsym (libcx_handle, "_waitpid");
+      /* If not available, falls back to waitpid() of kLIBC.  */
+      if (waitpid_pfn == NULL)
+        waitpid_pfn = waitpid;
+    }
+
+  return waitpid_pfn (pid, statusp, options);
+}
+
+# define waitpid klibc_waitpid
+#endif
+
 
 /* Type of an entry in the slaves array.
    The 'used' bit determines whether this entry is currently in use.
