@@ -1,5 +1,5 @@
 /* markup.c -- simple XML-like parser
-   Copyright (C) 2015-2016 Free Software Foundation, Inc.
+   Copyright (C) 2015-2024 Free Software Foundation, Inc.
 
    This file is not part of the GNU gettext program, but is used with
    GNU gettext.
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with GLib; see the file COPYING.LIB.  If not,
- * see <http://www.gnu.org/licenses/>.
+ * see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -48,6 +48,7 @@
 #include "unistr.h"
 #include "xalloc.h"
 #include "xvasprintf.h"
+#include "xstrerror.h"
 
 #define _(s) gettext(s)
 
@@ -66,7 +67,7 @@
  * the parser may accept documents that an XML parser would not.
  * However, XML documents which are not well-formed (which is a weaker
  * condition than being valid.  See the XML specification
- * <http://www.w3.org/TR/REC-xml/> for definitions of these terms.)
+ * <https://www.w3.org/TR/REC-xml/> for definitions of these terms.)
  * are not considered valid GMarkup documents.
  *
  * Simplifications to XML include:
@@ -361,7 +362,7 @@ slow_name_validate (markup_parse_context_ty *context, const char *name)
   const char *p = name;
   ucs4_t uc;
 
-  if (u8_check ((uint8_t *) name, strlen (name)) != NULL)
+  if (u8_check ((const uint8_t *) name, strlen (name)) != NULL)
     {
       emit_error (context, _("invalid UTF-8 sequence"));
       return false;
@@ -371,7 +372,7 @@ slow_name_validate (markup_parse_context_ty *context, const char *name)
         || (!IS_COMMON_NAME_END_CHAR (*p)
             && (*p == '_'
                 || *p == ':'
-                || (u8_mbtouc (&uc, (uint8_t *) name, strlen (name)) > 0
+                || (u8_mbtouc (&uc, (const uint8_t *) name, strlen (name)) > 0
                     && uc_is_alpha (uc))))))
     {
       char *error_text = xasprintf (_("'%s' is not a valid name: %c"),
@@ -381,9 +382,9 @@ slow_name_validate (markup_parse_context_ty *context, const char *name)
       return false;
     }
 
-  for (p = (char *) u8_next (&uc, (uint8_t *) name);
+  for (p = (const char *) u8_next (&uc, (const uint8_t *) name);
        p != NULL;
-       p = (char *) u8_next (&uc, (uint8_t *) p))
+       p = (const char *) u8_next (&uc, (const uint8_t *) p))
     {
       /* is_name_char */
       if (!(c_isalnum (*p) ||
@@ -522,7 +523,7 @@ unescape_string_inplace (markup_parse_context_ty *context,
                   char *error_text =
                     xasprintf (_("invalid character reference: %s"),
                                errno != 0
-                               ? strerror (errno)
+                               ? xstrerror (NULL, errno)
                                : _("not a valid number specification"));
                   emit_error (context, error_text);
                   free (error_text);
