@@ -1,9 +1,9 @@
 /* Dummy replacement for part of the public API of the libtextstyle library.
-   Copyright (C) 2006-2007, 2019-2022 Free Software Foundation, Inc.
+   Copyright (C) 2006-2007, 2019-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -30,9 +30,13 @@
 #ifndef _TEXTSTYLE_H
 #define _TEXTSTYLE_H
 
+/* This file uses _GL_ATTRIBUTE_MAYBE_UNUSED, HAVE_TCDRAIN.  */
+#if !_GL_CONFIG_H_INCLUDED
+ #error "Please include config.h first."
+#endif
+
 #include <errno.h>
 #include <stdarg.h>
-#include <stdbool.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -48,7 +52,7 @@
    _GL_ATTRIBUTE_SPEC_PRINTF_STANDARD  */
 /* __gnu_printf__ is supported in GCC >= 4.4.  */
 #ifndef _GL_ATTRIBUTE_SPEC_PRINTF_STANDARD
-# if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 4)
+# if (__GNUC__ + (__GNUC_MINOR__ >= 4) > 4) && !defined __clang__
 #  define _GL_ATTRIBUTE_SPEC_PRINTF_STANDARD __gnu_printf__
 # else
 #  define _GL_ATTRIBUTE_SPEC_PRINTF_STANDARD __printf__
@@ -228,6 +232,12 @@ styled_ostream_flush_to_current_style (_GL_ATTRIBUTE_MAYBE_UNUSED styled_ostream
 {
 }
 
+static inline bool
+is_instance_of_styled_ostream (_GL_ATTRIBUTE_MAYBE_UNUSED ostream_t stream)
+{
+  return false;
+}
+
 /* -------------------------- From file-ostream.h -------------------------- */
 
 typedef ostream_t file_ostream_t;
@@ -236,12 +246,24 @@ typedef ostream_t file_ostream_t;
 #define file_ostream_flush ostream_flush
 #define file_ostream_free ostream_free
 
+static inline FILE *
+file_ostream_get_stdio_stream (file_ostream_t stream)
+{
+  return stream;
+}
+
 static inline file_ostream_t
 file_ostream_create (FILE *fp)
 {
   return fp;
 }
 
+static inline bool
+is_instance_of_file_ostream (_GL_ATTRIBUTE_MAYBE_UNUSED ostream_t stream)
+{
+  return true;
+}
+
 /* --------------------------- From fd-ostream.h --------------------------- */
 
 typedef ostream_t fd_ostream_t;
@@ -250,6 +272,24 @@ typedef ostream_t fd_ostream_t;
 #define fd_ostream_flush ostream_flush
 #define fd_ostream_free ostream_free
 
+static inline int
+fd_ostream_get_descriptor (fd_ostream_t stream)
+{
+  return fileno (stream);
+}
+
+static inline const char *
+fd_ostream_get_filename (_GL_ATTRIBUTE_MAYBE_UNUSED fd_ostream_t stream)
+{
+  return NULL;
+}
+
+static inline bool
+fd_ostream_is_buffered (_GL_ATTRIBUTE_MAYBE_UNUSED fd_ostream_t stream)
+{
+  return false;
+}
+
 static inline fd_ostream_t
 fd_ostream_create (int fd, _GL_ATTRIBUTE_MAYBE_UNUSED const char *filename,
                    _GL_ATTRIBUTE_MAYBE_UNUSED bool buffered)
@@ -262,6 +302,12 @@ fd_ostream_create (int fd, _GL_ATTRIBUTE_MAYBE_UNUSED const char *filename,
     return fdopen (fd, "w");
 }
 
+static inline bool
+is_instance_of_fd_ostream (ostream_t stream)
+{
+  return fileno (stream) >= 0;
+}
+
 /* -------------------------- From term-ostream.h -------------------------- */
 
 typedef int term_color_t;
@@ -291,12 +337,37 @@ typedef enum
   UNDERLINE_DEFAULT = UNDERLINE_OFF
 } term_underline_t;
 
+typedef enum
+{
+  TTYCTL_AUTO = 0,  /* Automatic best-possible choice.  */
+  TTYCTL_NONE,      /* No control.
+                       Result: Garbled output can occur, and the terminal can
+                       be left in any state when the program is interrupted.  */
+  TTYCTL_PARTIAL,   /* Signal handling.
+                       Result: Garbled output can occur, but the terminal will
+                       be left in the default state when the program is
+                       interrupted.  */
+  TTYCTL_FULL       /* Signal handling and disabling echo and flush-upon-signal.
+                       Result: No garbled output, and the terminal will
+                       be left in the default state when the program is
+                       interrupted.  */
+} ttyctl_t;
+
 typedef ostream_t term_ostream_t;
 
 #define term_ostream_write_mem ostream_write_mem
 #define term_ostream_flush ostream_flush
 #define term_ostream_free ostream_free
 
+static inline term_color_t
+term_ostream_rgb_to_color (_GL_ATTRIBUTE_MAYBE_UNUSED term_ostream_t stream,
+                           _GL_ATTRIBUTE_MAYBE_UNUSED int red,
+                           _GL_ATTRIBUTE_MAYBE_UNUSED int green,
+                           _GL_ATTRIBUTE_MAYBE_UNUSED int blue)
+{
+  return COLOR_DEFAULT;
+}
+
 static inline term_color_t
 term_ostream_get_color (_GL_ATTRIBUTE_MAYBE_UNUSED term_ostream_t stream)
 {
@@ -382,21 +453,20 @@ term_ostream_flush_to_current_style (term_ostream_t stream)
   fflush (stream);
 }
 
-typedef enum
+#define term_ostream_get_descriptor fd_ostream_get_descriptor
+#define term_ostream_get_filename fd_ostream_get_filename
+
+static inline ttyctl_t
+term_ostream_get_tty_control (_GL_ATTRIBUTE_MAYBE_UNUSED term_ostream_t stream)
 {
-  TTYCTL_AUTO = 0,  /* Automatic best-possible choice.  */
-  TTYCTL_NONE,      /* No control.
-                       Result: Garbled output can occur, and the terminal can
-                       be left in any state when the program is interrupted.  */
-  TTYCTL_PARTIAL,   /* Signal handling.
-                       Result: Garbled output can occur, but the terminal will
-                       be left in the default state when the program is
-                       interrupted.  */
-  TTYCTL_FULL       /* Signal handling and disabling echo and flush-upon-signal.
-                       Result: No garbled output, and the the terminal will
-                       be left in the default state when the program is
-                       interrupted.  */
-} ttyctl_t;
+  return TTYCTL_NONE;
+}
+
+static inline ttyctl_t
+term_ostream_get_effective_tty_control (_GL_ATTRIBUTE_MAYBE_UNUSED term_ostream_t stream)
+{
+  return TTYCTL_NONE;
+}
 
 static inline term_ostream_t
 term_ostream_create (int fd, const char *filename,
@@ -405,6 +475,95 @@ term_ostream_create (int fd, const char *filename,
   return fd_ostream_create (fd, filename, true);
 }
 
+#define is_instance_of_term_ostream is_instance_of_fd_ostream
+
+/* ------------------------- From memory-ostream.h ------------------------- */
+
+typedef ostream_t memory_ostream_t;
+
+#define memory_ostream_write_mem ostream_write_mem
+#define memory_ostream_flush ostream_flush
+#define memory_ostream_free ostream_free
+
+static inline void
+memory_ostream_contents (_GL_ATTRIBUTE_MAYBE_UNUSED memory_ostream_t stream,
+                         const void **bufp, size_t *buflenp)
+{
+  *bufp = NULL;
+  *buflenp = 0;
+}
+
+static inline memory_ostream_t
+memory_ostream_create (void)
+{
+  /* Not supported without the real libtextstyle.  */
+  abort ();
+  return NULL;
+}
+
+static inline bool
+is_instance_of_memory_ostream (_GL_ATTRIBUTE_MAYBE_UNUSED ostream_t stream)
+{
+  return false;
+}
+
+/* -------------------------- From html-ostream.h -------------------------- */
+
+typedef ostream_t html_ostream_t;
+
+#define html_ostream_write_mem ostream_write_mem
+#define html_ostream_flush ostream_flush
+#define html_ostream_free ostream_free
+
+static inline void
+html_ostream_begin_span (_GL_ATTRIBUTE_MAYBE_UNUSED html_ostream_t stream,
+                         _GL_ATTRIBUTE_MAYBE_UNUSED const char *classname)
+{
+}
+
+static inline void
+html_ostream_end_span (_GL_ATTRIBUTE_MAYBE_UNUSED html_ostream_t stream,
+                       _GL_ATTRIBUTE_MAYBE_UNUSED const char *classname)
+{
+}
+
+static inline const char *
+html_ostream_get_hyperlink_ref (_GL_ATTRIBUTE_MAYBE_UNUSED html_ostream_t stream)
+{
+  return NULL;
+}
+
+static inline void
+html_ostream_set_hyperlink_ref (_GL_ATTRIBUTE_MAYBE_UNUSED html_ostream_t stream,
+                                _GL_ATTRIBUTE_MAYBE_UNUSED const char *ref)
+{
+}
+
+static inline void
+html_ostream_flush_to_current_style (_GL_ATTRIBUTE_MAYBE_UNUSED html_ostream_t stream)
+{
+}
+
+static inline ostream_t
+html_ostream_get_destination (_GL_ATTRIBUTE_MAYBE_UNUSED html_ostream_t stream)
+{
+  return NULL;
+}
+
+static inline html_ostream_t
+html_ostream_create (ostream_t destination)
+{
+  /* Not supported without the real libtextstyle.  */
+  abort ();
+  return NULL;
+}
+
+static inline bool
+is_instance_of_html_ostream (_GL_ATTRIBUTE_MAYBE_UNUSED ostream_t stream)
+{
+  return false;
+}
+
 /* ----------------------- From term-styled-ostream.h ----------------------- */
 
 typedef styled_ostream_t term_styled_ostream_t;
@@ -419,6 +578,18 @@ typedef styled_ostream_t term_styled_ostream_t;
 #define term_styled_ostream_set_hyperlink styled_ostream_set_hyperlink
 #define term_styled_ostream_flush_to_current_style styled_ostream_flush_to_current_style
 
+static inline term_ostream_t
+term_styled_ostream_get_destination (term_styled_ostream_t stream)
+{
+  return stream;
+}
+
+static inline const char *
+term_styled_ostream_get_css_filename (_GL_ATTRIBUTE_MAYBE_UNUSED term_styled_ostream_t stream)
+{
+  return NULL;
+}
+
 static inline term_styled_ostream_t
 term_styled_ostream_create (int fd, const char *filename,
                             _GL_ATTRIBUTE_MAYBE_UNUSED ttyctl_t tty_control,
@@ -427,18 +598,96 @@ term_styled_ostream_create (int fd, const char *filename,
   return fd_ostream_create (fd, filename, true);
 }
 
+#define is_instance_of_term_styled_ostream is_instance_of_term_ostream
+
 /* ----------------------- From html-styled-ostream.h ----------------------- */
 
 typedef styled_ostream_t html_styled_ostream_t;
 
+#define html_styled_ostream_write_mem ostream_write_mem
+#define html_styled_ostream_flush ostream_flush
+#define html_styled_ostream_free ostream_free
+#define html_styled_ostream_begin_use_class styled_ostream_begin_use_class
+#define html_styled_ostream_end_use_class styled_ostream_end_use_class
+#define html_styled_ostream_get_hyperlink_ref styled_ostream_get_hyperlink_ref
+#define html_styled_ostream_get_hyperlink_id styled_ostream_get_hyperlink_id
+#define html_styled_ostream_set_hyperlink styled_ostream_set_hyperlink
+#define html_styled_ostream_flush_to_current_style styled_ostream_flush_to_current_style
+
+static inline ostream_t
+html_styled_ostream_get_destination (_GL_ATTRIBUTE_MAYBE_UNUSED html_styled_ostream_t stream)
+{
+  return NULL;
+}
+
+static inline html_ostream_t
+html_styled_ostream_get_html_destination (_GL_ATTRIBUTE_MAYBE_UNUSED html_styled_ostream_t stream)
+{
+  return NULL;
+}
+
+static inline const char *
+html_styled_ostream_get_css_filename (_GL_ATTRIBUTE_MAYBE_UNUSED html_styled_ostream_t stream)
+{
+  return NULL;
+}
+
 static inline html_styled_ostream_t
 html_styled_ostream_create (_GL_ATTRIBUTE_MAYBE_UNUSED ostream_t destination,
                             _GL_ATTRIBUTE_MAYBE_UNUSED const char *css_filename)
 {
+  /* Not supported without the real libtextstyle.  */
   abort ();
   return NULL;
 }
 
+static inline bool
+is_instance_of_html_styled_ostream (_GL_ATTRIBUTE_MAYBE_UNUSED ostream_t stream)
+{
+  return false;
+}
+
+/* ----------------------- From noop-styled-ostream.h ----------------------- */
+
+typedef styled_ostream_t noop_styled_ostream_t;
+
+#define noop_styled_ostream_write_mem ostream_write_mem
+#define noop_styled_ostream_flush ostream_flush
+#define noop_styled_ostream_free ostream_free
+#define noop_styled_ostream_begin_use_class styled_ostream_begin_use_class
+#define noop_styled_ostream_end_use_class styled_ostream_end_use_class
+#define noop_styled_ostream_get_hyperlink_ref styled_ostream_get_hyperlink_ref
+#define noop_styled_ostream_get_hyperlink_id styled_ostream_get_hyperlink_id
+#define noop_styled_ostream_set_hyperlink styled_ostream_set_hyperlink
+#define noop_styled_ostream_flush_to_current_style styled_ostream_flush_to_current_style
+
+static inline ostream_t
+noop_styled_ostream_get_destination (noop_styled_ostream_t stream)
+{
+  return stream;
+}
+
+static inline bool
+noop_styled_ostream_is_owning_destination (_GL_ATTRIBUTE_MAYBE_UNUSED noop_styled_ostream_t stream)
+{
+  return true;
+}
+
+static inline noop_styled_ostream_t
+noop_styled_ostream_create (ostream_t destination, bool pass_ownership)
+{
+  if (!pass_ownership)
+    /* Not supported without the real libtextstyle.  */
+    abort ();
+  return destination;
+}
+
+static inline bool
+is_instance_of_noop_styled_ostream (_GL_ATTRIBUTE_MAYBE_UNUSED ostream_t stream)
+{
+  return false;
+}
+
 /* ------------------------------ From color.h ------------------------------ */
 
 #define color_test_mode false
@@ -462,6 +711,7 @@ handle_style_option (_GL_ATTRIBUTE_MAYBE_UNUSED const char *option)
 static inline void
 print_color_test (void)
 {
+  /* Not supported without the real libtextstyle.  */
   abort ();
 }
 
