@@ -1,5 +1,5 @@
 /* Format strings.
-   Copyright (C) 2001-2009, 2015-2016 Free Software Foundation, Inc.
+   Copyright (C) 2001-2010, 2012-2013, 2015, 2019-2020, 2023 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -36,32 +36,35 @@ struct formatstring_parser *formatstring_parsers[NFORMATS] =
 {
   /* format_c */                &formatstring_c,
   /* format_objc */             &formatstring_objc,
-  /* format_sh */               &formatstring_sh,
+  /* format_cplusplus_brace */  &formatstring_cplusplus_brace,
   /* format_python */           &formatstring_python,
   /* format_python_brace */     &formatstring_python_brace,
+  /* format_java */             &formatstring_java,
+  /* format_java_printf */      &formatstring_java_printf,
+  /* format_csharp */           &formatstring_csharp,
+  /* format_javascript */       &formatstring_javascript,
+  /* format_scheme */           &formatstring_scheme,
   /* format_lisp */             &formatstring_lisp,
   /* format_elisp */            &formatstring_elisp,
   /* format_librep */           &formatstring_librep,
-  /* format_scheme */           &formatstring_scheme,
-  /* format_smalltalk */        &formatstring_smalltalk,
-  /* format_java */             &formatstring_java,
-  /* format_csharp */           &formatstring_csharp,
+  /* format_ruby */             &formatstring_ruby,
+  /* format_sh */               &formatstring_sh,
   /* format_awk */              &formatstring_awk,
+  /* format_lua */              &formatstring_lua,
   /* format_pascal */           &formatstring_pascal,
-  /* format_ycp */              &formatstring_ycp,
+  /* format_smalltalk */        &formatstring_smalltalk,
+  /* format_qt */               &formatstring_qt,
+  /* format_qt_plural */        &formatstring_qt_plural,
+  /* format_kde */              &formatstring_kde,
+  /* format_kde_kuit */         &formatstring_kde_kuit,
+  /* format_boost */            &formatstring_boost,
   /* format_tcl */              &formatstring_tcl,
   /* format_perl */             &formatstring_perl,
   /* format_perl_brace */       &formatstring_perl_brace,
   /* format_php */              &formatstring_php,
   /* format_gcc_internal */     &formatstring_gcc_internal,
   /* format_gfc_internal */     &formatstring_gfc_internal,
-  /* format_qt */               &formatstring_qt,
-  /* format_qt_plural */        &formatstring_qt_plural,
-  /* format_kde */              &formatstring_kde,
-  /* format_kde_kuit */         &formatstring_kde_kuit,
-  /* format_boost */            &formatstring_boost,
-  /* format_lua */              &formatstring_lua,
-  /* format_javascript */       &formatstring_javascript
+  /* format_ycp */              &formatstring_ycp
 };
 
 /* Check whether both formats strings contain compatible format
@@ -72,7 +75,7 @@ check_msgid_msgstr_format_i (const char *msgid, const char *msgid_plural,
                              size_t i,
                              struct argument_range range,
                              const struct plural_distribution *distribution,
-                             formatstring_error_logger_t error_logger)
+                             formatstring_error_logger_t error_logger, void *error_logger_data)
 {
   int seen_errors = 0;
 
@@ -142,15 +145,16 @@ check_msgid_msgstr_format_i (const char *msgid, const char *msgid_plural,
 
               if (parser->check (msgid_descr, msgstr_descr,
                                  strict_checking,
-                                 error_logger, pretty_msgid, pretty_msgstr))
+                                 error_logger, error_logger_data,
+                                 pretty_msgid, pretty_msgstr))
                 seen_errors++;
 
               parser->free (msgstr_descr);
             }
           else
             {
-              error_logger (_("\
-'%s' is not a valid %s format string, unlike '%s'. Reason: %s"),
+              error_logger (error_logger_data,
+                            _("'%s' is not a valid %s format string, unlike '%s'. Reason: %s"),
                             pretty_msgstr, format_language_pretty[i],
                             pretty_msgid, invalid_reason);
               seen_errors++;
@@ -175,7 +179,7 @@ check_msgid_msgstr_format (const char *msgid, const char *msgid_plural,
                            const enum is_format is_format[NFORMATS],
                            struct argument_range range,
                            const struct plural_distribution *distribution,
-                           formatstring_error_logger_t error_logger)
+                           formatstring_error_logger_t error_logger, void *error_logger_data)
 {
   int seen_errors = 0;
   size_t i;
@@ -192,7 +196,7 @@ check_msgid_msgstr_format (const char *msgid, const char *msgid_plural,
                                                   msgstr, msgstr_len, i,
                                                   range,
                                                   distribution,
-                                                  error_logger);
+                                                  error_logger, error_logger_data);
 
   return seen_errors;
 }
