@@ -1,5 +1,5 @@
-/* Reading PO files.
-   Copyright (C) 1995-1998, 2000-2003, 2005-2006, 2008-2009, 2014-2016, 2023 Free Software Foundation, Inc.
+/* Reading textual message catalogs (such as PO files).
+   Copyright (C) 1995-2024 Free Software Foundation, Inc.
    This file was written by Peter Miller <millerp@canb.auug.org.au>
 
    This program is free software: you can redistribute it and/or modify
@@ -26,10 +26,11 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "open-catalog.h"
 #include "po-charset.h"
-#include "po-xerror.h"
+#include "read-po-lex.h"
+#include "xerror-handler.h"
 #include "xalloc.h"
+#include "read-catalog-special.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -39,17 +40,18 @@
 /* Inline functions to invoke the methods.  */
 
 static inline void
-call_set_domain (struct default_catalog_reader_ty *this, char *name)
+call_set_domain (struct default_catalog_reader_ty *dcatr,
+                 char *name, lex_pos_ty *name_pos)
 {
   default_catalog_reader_class_ty *methods =
-    (default_catalog_reader_class_ty *) this->methods;
+    (default_catalog_reader_class_ty *) dcatr->methods;
 
   if (methods->set_domain)
-    methods->set_domain (this, name);
+    methods->set_domain (dcatr, name, name_pos);
 }
 
 static inline void
-call_add_message (struct default_catalog_reader_ty *this,
+call_add_message (struct default_catalog_reader_ty *dcatr,
                   char *msgctxt,
                   char *msgid, lex_pos_ty *msgid_pos, char *msgid_plural,
                   char *msgstr, size_t msgstr_len, lex_pos_ty *msgstr_pos,
@@ -57,10 +59,10 @@ call_add_message (struct default_catalog_reader_ty *this,
                   bool force_fuzzy, bool obsolete)
 {
   default_catalog_reader_class_ty *methods =
-    (default_catalog_reader_class_ty *) this->methods;
+    (default_catalog_reader_class_ty *) dcatr->methods;
 
   if (methods->add_message)
-    methods->add_message (this, msgctxt,
+    methods->add_message (dcatr, msgctxt,
                           msgid, msgid_pos, msgid_plural,
                           msgstr, msgstr_len, msgstr_pos,
                           prev_msgctxt, prev_msgid, prev_msgid_plural,
@@ -68,15 +70,15 @@ call_add_message (struct default_catalog_reader_ty *this,
 }
 
 static inline void
-call_frob_new_message (struct default_catalog_reader_ty *this, message_ty *mp,
+call_frob_new_message (struct default_catalog_reader_ty *dcatr, message_ty *mp,
                        const lex_pos_ty *msgid_pos,
                        const lex_pos_ty *msgstr_pos)
 {
   default_catalog_reader_class_ty *methods =
-    (default_catalog_reader_class_ty *) this->methods;
+    (default_catalog_reader_class_ty *) dcatr->methods;
 
   if (methods->frob_new_message)
-    methods->frob_new_message (this, mp, msgid_pos, msgstr_pos);
+    methods->frob_new_message (dcatr, mp, msgid_pos, msgstr_pos);
 }
 
 
@@ -89,149 +91,150 @@ call_frob_new_message (struct default_catalog_reader_ty *this, message_ty *mp,
 
 /* Prepare for first message.  */
 void
-default_constructor (abstract_catalog_reader_ty *that)
+default_constructor (abstract_catalog_reader_ty *catr)
 {
-  default_catalog_reader_ty *this = (default_catalog_reader_ty *) that;
+  default_catalog_reader_ty *dcatr = (default_catalog_reader_ty *) catr;
   size_t i;
 
-  this->domain = MESSAGE_DOMAIN_DEFAULT;
-  this->comment = NULL;
-  this->comment_dot = NULL;
-  this->filepos_count = 0;
-  this->filepos = NULL;
-  this->is_fuzzy = false;
+  dcatr->domain = MESSAGE_DOMAIN_DEFAULT;
+  dcatr->comment = NULL;
+  dcatr->comment_dot = NULL;
+  dcatr->filepos_count = 0;
+  dcatr->filepos = NULL;
+  dcatr->is_fuzzy = false;
   for (i = 0; i < NFORMATS; i++)
-    this->is_format[i] = undecided;
-  this->range.min = -1;
-  this->range.max = -1;
-  this->do_wrap = undecided;
+    dcatr->is_format[i] = undecided;
+  dcatr->range.min = -1;
+  dcatr->range.max = -1;
+  dcatr->do_wrap = undecided;
   for (i = 0; i < NSYNTAXCHECKS; i++)
-    this->do_syntax_check[i] = undecided;
+    dcatr->do_syntax_check[i] = undecided;
 }
 
 
 void
-default_destructor (abstract_catalog_reader_ty *that)
+default_destructor (abstract_catalog_reader_ty *catr)
 {
-  default_catalog_reader_ty *this = (default_catalog_reader_ty *) that;
+  default_catalog_reader_ty *dcatr = (default_catalog_reader_ty *) catr;
   size_t j;
 
-  /* Do not free this->mdlp and this->mlp.  */
-  if (this->handle_comments)
+  /* Do not free dcatr->mdlp and dcatr->mlp.  */
+  if (dcatr->handle_comments)
     {
-      if (this->comment != NULL)
-        string_list_free (this->comment);
-      if (this->comment_dot != NULL)
-        string_list_free (this->comment_dot);
+      if (dcatr->comment != NULL)
+        string_list_free (dcatr->comment);
+      if (dcatr->comment_dot != NULL)
+        string_list_free (dcatr->comment_dot);
     }
 
-  for (j = 0; j < this->filepos_count; ++j)
-    free ((char *) this->filepos[j].file_name);
-  if (this->filepos != NULL)
-    free (this->filepos);
+  for (j = 0; j < dcatr->filepos_count; ++j)
+    free ((char *) dcatr->filepos[j].file_name);
+  if (dcatr->filepos != NULL)
+    free (dcatr->filepos);
 }
 
 
 void
-default_parse_brief (abstract_catalog_reader_ty *that)
+default_parse_brief (abstract_catalog_reader_ty *catr)
 {
-  /* We need to parse comments, because even if this->handle_comments
+  /* We need to parse comments, because even if dcatr->handle_comments
      is false, we need to know which messages are fuzzy.  */
-  po_lex_pass_comments (true);
+  catr->pass_comments = true;
 }
 
 
 void
-default_parse_debrief (abstract_catalog_reader_ty *that)
+default_parse_debrief (abstract_catalog_reader_ty *catr)
 {
 }
 
 
 /* Add the accumulated comments to the message.  */
 static void
-default_copy_comment_state (default_catalog_reader_ty *this, message_ty *mp)
+default_copy_comment_state (default_catalog_reader_ty *dcatr, message_ty *mp)
 {
   size_t j, i;
 
-  if (this->handle_comments)
+  if (dcatr->handle_comments)
     {
-      if (this->comment != NULL)
-        for (j = 0; j < this->comment->nitems; ++j)
-          message_comment_append (mp, this->comment->item[j]);
-      if (this->comment_dot != NULL)
-        for (j = 0; j < this->comment_dot->nitems; ++j)
-          message_comment_dot_append (mp, this->comment_dot->item[j]);
+      if (dcatr->comment != NULL)
+        for (j = 0; j < dcatr->comment->nitems; ++j)
+          message_comment_append (mp, dcatr->comment->item[j]);
+      if (dcatr->comment_dot != NULL)
+        for (j = 0; j < dcatr->comment_dot->nitems; ++j)
+          message_comment_dot_append (mp, dcatr->comment_dot->item[j]);
     }
-  for (j = 0; j < this->filepos_count; ++j)
+  for (j = 0; j < dcatr->filepos_count; ++j)
     {
       lex_pos_ty *pp;
 
-      pp = &this->filepos[j];
+      pp = &dcatr->filepos[j];
       message_comment_filepos (mp, pp->file_name, pp->line_number);
     }
-  mp->is_fuzzy = this->is_fuzzy;
+  mp->is_fuzzy = dcatr->is_fuzzy;
   for (i = 0; i < NFORMATS; i++)
-    mp->is_format[i] = this->is_format[i];
-  mp->range = this->range;
-  mp->do_wrap = this->do_wrap;
+    mp->is_format[i] = dcatr->is_format[i];
+  mp->range = dcatr->range;
+  mp->do_wrap = dcatr->do_wrap;
   for (i = 0; i < NSYNTAXCHECKS; i++)
-    mp->do_syntax_check[i] = this->do_syntax_check[i];
+    mp->do_syntax_check[i] = dcatr->do_syntax_check[i];
 }
 
 
 static void
-default_reset_comment_state (default_catalog_reader_ty *this)
+default_reset_comment_state (default_catalog_reader_ty *dcatr)
 {
   size_t j, i;
 
-  if (this->handle_comments)
+  if (dcatr->handle_comments)
     {
-      if (this->comment != NULL)
+      if (dcatr->comment != NULL)
         {
-          string_list_free (this->comment);
-          this->comment = NULL;
+          string_list_free (dcatr->comment);
+          dcatr->comment = NULL;
         }
-      if (this->comment_dot != NULL)
+      if (dcatr->comment_dot != NULL)
         {
-          string_list_free (this->comment_dot);
-          this->comment_dot = NULL;
+          string_list_free (dcatr->comment_dot);
+          dcatr->comment_dot = NULL;
         }
     }
-  for (j = 0; j < this->filepos_count; ++j)
-    free ((char *) this->filepos[j].file_name);
-  if (this->filepos != NULL)
-    free (this->filepos);
-  this->filepos_count = 0;
-  this->filepos = NULL;
-  this->is_fuzzy = false;
+  for (j = 0; j < dcatr->filepos_count; ++j)
+    free ((char *) dcatr->filepos[j].file_name);
+  if (dcatr->filepos != NULL)
+    free (dcatr->filepos);
+  dcatr->filepos_count = 0;
+  dcatr->filepos = NULL;
+  dcatr->is_fuzzy = false;
   for (i = 0; i < NFORMATS; i++)
-    this->is_format[i] = undecided;
-  this->range.min = -1;
-  this->range.max = -1;
-  this->do_wrap = undecided;
+    dcatr->is_format[i] = undecided;
+  dcatr->range.min = -1;
+  dcatr->range.max = -1;
+  dcatr->do_wrap = undecided;
   for (i = 0; i < NSYNTAXCHECKS; i++)
-    this->do_syntax_check[i] = undecided;
+    dcatr->do_syntax_check[i] = undecided;
 }
 
 
 /* Process 'domain' directive from .po file.  */
 void
-default_directive_domain (abstract_catalog_reader_ty *that, char *name)
+default_directive_domain (abstract_catalog_reader_ty *catr,
+                          char *name, lex_pos_ty *name_pos)
 {
-  default_catalog_reader_ty *this = (default_catalog_reader_ty *) that;
+  default_catalog_reader_ty *dcatr = (default_catalog_reader_ty *) catr;
 
-  call_set_domain (this, name);
+  call_set_domain (dcatr, name, name_pos);
 
   /* If there are accumulated comments, throw them away, they are
      probably part of the file header, or about the domain directive,
      and will be unrelated to the next message.  */
-  default_reset_comment_state (this);
+  default_reset_comment_state (dcatr);
 }
 
 
 /* Process ['msgctxt'/]'msgid'/'msgstr' pair from .po file.  */
 void
-default_directive_message (abstract_catalog_reader_ty *that,
+default_directive_message (abstract_catalog_reader_ty *catr,
                            char *msgctxt,
                            char *msgid,
                            lex_pos_ty *msgid_pos,
@@ -242,70 +245,70 @@ default_directive_message (abstract_catalog_reader_ty *that,
                            char *prev_msgid, char *prev_msgid_plural,
                            bool force_fuzzy, bool obsolete)
 {
-  default_catalog_reader_ty *this = (default_catalog_reader_ty *) that;
+  default_catalog_reader_ty *dcatr = (default_catalog_reader_ty *) catr;
 
-  call_add_message (this, msgctxt, msgid, msgid_pos, msgid_plural,
+  call_add_message (dcatr, msgctxt, msgid, msgid_pos, msgid_plural,
                     msgstr, msgstr_len, msgstr_pos,
                     prev_msgctxt, prev_msgid, prev_msgid_plural,
                     force_fuzzy, obsolete);
 
   /* Prepare for next message.  */
-  default_reset_comment_state (this);
+  default_reset_comment_state (dcatr);
 }
 
 
 void
-default_comment (abstract_catalog_reader_ty *that, const char *s)
+default_comment (abstract_catalog_reader_ty *catr, const char *s)
 {
-  default_catalog_reader_ty *this = (default_catalog_reader_ty *) that;
+  default_catalog_reader_ty *dcatr = (default_catalog_reader_ty *) catr;
 
-  if (this->handle_comments)
+  if (dcatr->handle_comments)
     {
-      if (this->comment == NULL)
-        this->comment = string_list_alloc ();
-      string_list_append (this->comment, s);
+      if (dcatr->comment == NULL)
+        dcatr->comment = string_list_alloc ();
+      string_list_append (dcatr->comment, s);
     }
 }
 
 
 void
-default_comment_dot (abstract_catalog_reader_ty *that, const char *s)
+default_comment_dot (abstract_catalog_reader_ty *catr, const char *s)
 {
-  default_catalog_reader_ty *this = (default_catalog_reader_ty *) that;
+  default_catalog_reader_ty *dcatr = (default_catalog_reader_ty *) catr;
 
-  if (this->handle_comments)
+  if (dcatr->handle_comments)
     {
-      if (this->comment_dot == NULL)
-        this->comment_dot = string_list_alloc ();
-      string_list_append (this->comment_dot, s);
+      if (dcatr->comment_dot == NULL)
+        dcatr->comment_dot = string_list_alloc ();
+      string_list_append (dcatr->comment_dot, s);
     }
 }
 
 
 void
-default_comment_filepos (abstract_catalog_reader_ty *that,
-                         const char *name, size_t line)
+default_comment_filepos (abstract_catalog_reader_ty *catr,
+                         const char *file_name, size_t line_number)
 {
-  default_catalog_reader_ty *this = (default_catalog_reader_ty *) that;
+  default_catalog_reader_ty *dcatr = (default_catalog_reader_ty *) catr;
   size_t nbytes;
   lex_pos_ty *pp;
 
-  nbytes = (this->filepos_count + 1) * sizeof (this->filepos[0]);
-  this->filepos = xrealloc (this->filepos, nbytes);
-  pp = &this->filepos[this->filepos_count++];
-  pp->file_name = xstrdup (name);
-  pp->line_number = line;
+  nbytes = (dcatr->filepos_count + 1) * sizeof (dcatr->filepos[0]);
+  dcatr->filepos = xrealloc (dcatr->filepos, nbytes);
+  pp = &dcatr->filepos[dcatr->filepos_count++];
+  pp->file_name = xstrdup (file_name);
+  pp->line_number = line_number;
 }
 
 
 /* Test for '#, fuzzy' comments and warn.  */
 void
-default_comment_special (abstract_catalog_reader_ty *that, const char *s)
+default_comment_special (abstract_catalog_reader_ty *catr, const char *s)
 {
-  default_catalog_reader_ty *this = (default_catalog_reader_ty *) that;
+  default_catalog_reader_ty *dcatr = (default_catalog_reader_ty *) catr;
 
-  po_parse_comment_special (s, &this->is_fuzzy, this->is_format, &this->range,
-                            &this->do_wrap, this->do_syntax_check);
+  parse_comment_special (s, &dcatr->is_fuzzy, dcatr->is_format, &dcatr->range,
+                         &dcatr->do_wrap, dcatr->do_syntax_check);
 }
 
 
@@ -313,23 +316,26 @@ default_comment_special (abstract_catalog_reader_ty *that, const char *s)
 
 
 void
-default_set_domain (default_catalog_reader_ty *this, char *name)
+default_set_domain (default_catalog_reader_ty *dcatr,
+                    char *name, lex_pos_ty *name_pos)
 {
-  if (this->allow_domain_directives)
+  if (dcatr->allow_domain_directives)
     /* Override current domain name.  Don't free memory.  */
-    this->domain = name;
+    dcatr->domain = name;
   else
     {
-      po_gram_error_at_line (&gram_pos,
-                             _("this file may not contain domain directives"));
+      dcatr->xeh->xerror (CAT_SEVERITY_ERROR, NULL,
+                          name_pos->file_name, name_pos->line_number, (size_t)(-1),
+                          false,
+                          _("this file may not contain domain directives"));
 
-      /* NAME was allocated in po-gram-gen.y but is not used anywhere.  */
+      /* NAME was allocated in read-po-gram.y but is not used anywhere.  */
       free (name);
     }
 }
 
 void
-default_add_message (default_catalog_reader_ty *this,
+default_add_message (default_catalog_reader_ty *dcatr,
                      char *msgctxt,
                      char *msgid,
                      lex_pos_ty *msgid_pos,
@@ -343,20 +349,20 @@ default_add_message (default_catalog_reader_ty *this,
 {
   message_ty *mp;
 
-  if (this->mdlp != NULL)
-    /* Select the appropriate sublist of this->mdlp.  */
-    this->mlp = msgdomain_list_sublist (this->mdlp, this->domain, true);
+  if (dcatr->mdlp != NULL)
+    /* Select the appropriate sublist of dcatr->mdlp.  */
+    dcatr->mlp = msgdomain_list_sublist (dcatr->mdlp, dcatr->domain, true);
 
-  if (this->allow_duplicates && msgid[0] != '\0')
+  if (dcatr->allow_duplicates && msgid[0] != '\0')
     /* Doesn't matter if this message ID has been seen before.  */
     mp = NULL;
   else
     /* See if this message ID has been seen before.  */
-    mp = message_list_search (this->mlp, msgctxt, msgid);
+    mp = message_list_search (dcatr->mlp, msgctxt, msgid);
 
   if (mp)
     {
-      if (!(this->allow_duplicates_if_same_msgstr
+      if (!(dcatr->allow_duplicates_if_same_msgstr
             && msgstr_len == mp->msgstr_len
             && memcmp (msgstr, mp->msgstr, msgstr_len) == 0))
         {
@@ -364,14 +370,18 @@ default_add_message (default_catalog_reader_ty *this,
              translations are equal or different.  This is for consistency
              with msgmerge, msgcat and others.  The user can use the
              msguniq program to get rid of duplicates.  */
-          po_xerror2 (PO_SEVERITY_ERROR,
-                      NULL, msgid_pos->file_name, msgid_pos->line_number,
-                      (size_t)(-1), false, _("duplicate message definition"),
-                      mp, NULL, 0, 0, false,
-                      _("this is the location of the first definition"));
+          dcatr->xeh->xerror2 (CAT_SEVERITY_ERROR,
+                               NULL,
+                               msgid_pos->file_name, msgid_pos->line_number, (size_t)(-1),
+                               false,
+                               _("duplicate message definition"),
+                               mp,
+                               NULL, 0, 0,
+                               false,
+                               _("this is the location of the first definition"));
         }
       /* We don't need the just constructed entries' parameter string
-         (allocated in po-gram-gen.y).  */
+         (allocated in read-po-gram.y).  */
       free (msgid);
       if (msgid_plural != NULL)
         free (msgid_plural);
@@ -386,7 +396,7 @@ default_add_message (default_catalog_reader_ty *this,
         free (prev_msgid_plural);
 
       /* Add the accumulated comments to the message.  */
-      default_copy_comment_state (this, mp);
+      default_copy_comment_state (dcatr, mp);
     }
   else
     {
@@ -402,13 +412,13 @@ default_add_message (default_catalog_reader_ty *this,
       mp->prev_msgid = prev_msgid;
       mp->prev_msgid_plural = prev_msgid_plural;
       mp->obsolete = obsolete;
-      default_copy_comment_state (this, mp);
+      default_copy_comment_state (dcatr, mp);
       if (force_fuzzy)
         mp->is_fuzzy = true;
 
-      call_frob_new_message (this, mp, msgid_pos, msgstr_pos);
+      call_frob_new_message (dcatr, mp, msgid_pos, msgstr_pos);
 
-      message_list_append (this->mlp, mp);
+      message_list_append (dcatr->mlp, mp);
     }
 }
 
@@ -441,10 +451,12 @@ static default_catalog_reader_class_ty default_methods =
 
 
 default_catalog_reader_ty *
-default_catalog_reader_alloc (default_catalog_reader_class_ty *method_table)
+default_catalog_reader_alloc (default_catalog_reader_class_ty *method_table,
+                              xerror_handler_ty xerror_handler)
 {
   return
-    (default_catalog_reader_ty *) catalog_reader_alloc (&method_table->super);
+    (default_catalog_reader_ty *)
+    catalog_reader_alloc (&method_table->super, xerror_handler);
 }
 
 
@@ -461,42 +473,27 @@ bool allow_duplicates = false;
 msgdomain_list_ty *
 read_catalog_stream (FILE *fp, const char *real_filename,
                      const char *logical_filename,
-                     catalog_input_format_ty input_syntax)
+                     catalog_input_format_ty input_syntax,
+                     xerror_handler_ty xerror_handler)
 {
-  default_catalog_reader_ty *pop;
+  default_catalog_reader_ty *dcatr;
   msgdomain_list_ty *mdlp;
 
-  pop = default_catalog_reader_alloc (&default_methods);
-  pop->handle_comments = true;
-  pop->allow_domain_directives = true;
-  pop->allow_duplicates = allow_duplicates;
-  pop->allow_duplicates_if_same_msgstr = false;
-  pop->file_name = real_filename;
-  pop->mdlp = msgdomain_list_alloc (!pop->allow_duplicates);
-  pop->mlp = msgdomain_list_sublist (pop->mdlp, pop->domain, true);
+  dcatr = default_catalog_reader_alloc (&default_methods, xerror_handler);
+  dcatr->pass_obsolete_entries = true;
+  dcatr->handle_comments = true;
+  dcatr->allow_domain_directives = true;
+  dcatr->allow_duplicates = allow_duplicates;
+  dcatr->allow_duplicates_if_same_msgstr = false;
+  dcatr->file_name = real_filename;
+  dcatr->mdlp = msgdomain_list_alloc (!dcatr->allow_duplicates);
+  dcatr->mlp = msgdomain_list_sublist (dcatr->mdlp, dcatr->domain, true);
   if (input_syntax->produces_utf8)
     /* We know a priori that input_syntax->parse convert strings to UTF-8.  */
-    pop->mdlp->encoding = po_charset_utf8;
-  po_lex_pass_obsolete_entries (true);
-  catalog_reader_parse ((abstract_catalog_reader_ty *) pop, fp, real_filename,
-                        logical_filename, input_syntax);
-  mdlp = pop->mdlp;
-  catalog_reader_free ((abstract_catalog_reader_ty *) pop);
+    dcatr->mdlp->encoding = po_charset_utf8;
+  catalog_reader_parse ((abstract_catalog_reader_ty *) dcatr, fp, real_filename,
+                        logical_filename, false, input_syntax);
+  mdlp = dcatr->mdlp;
+  catalog_reader_free ((abstract_catalog_reader_ty *) dcatr);
   return mdlp;
 }
-
-
-msgdomain_list_ty *
-read_catalog_file (const char *filename, catalog_input_format_ty input_syntax)
-{
-  char *real_filename;
-  FILE *fp = open_catalog_file (filename, &real_filename, true);
-  msgdomain_list_ty *result;
-
-  result = read_catalog_stream (fp, real_filename, filename, input_syntax);
-
-  if (fp != stdin)
-    fclose (fp);
-
-  return result;
-}
