@@ -1,5 +1,5 @@
 /* xgettext Scheme backend.
-   Copyright (C) 2004, 2006, 2015 Free Software Foundation, Inc.
+   Copyright (C) 2004-2024 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2004.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,13 +13,13 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 
 #include <stdio.h>
 
 #include "message.h"
-#include "xgettext.h"
+#include "xg-arglist-context.h"
 
 
 #ifdef __cplusplus
@@ -28,11 +28,13 @@ extern "C" {
 
 
 #define EXTENSIONS_SCHEME \
-  { "scm",       "Scheme"     },                                        \
+  { "scm",       "Guile"      },                                        \
 
 #define SCANNERS_SCHEME \
-  { "Scheme",           extract_scheme,                                   \
-                        &flag_table_scheme, &formatstring_scheme, NULL, NULL }, \
+  { "Scheme",           extract_scheme, NULL,                             \
+                        &flag_table_scheme, &formatstring_scheme, NULL }, \
+  { "Guile",            extract_guile, NULL,                              \
+                        &flag_table_scheme, &formatstring_scheme, NULL }, \
 
 /* Scan a Scheme file and add its translatable strings to mdlp.  */
 extern void extract_scheme (FILE *fp, const char *real_filename,
@@ -40,6 +42,12 @@ extern void extract_scheme (FILE *fp, const char *real_filename,
                             flag_context_list_table_ty *flag_table,
                             msgdomain_list_ty *mdlp);
 
+/* Scan a Guile file and add its translatable strings to mdlp.  */
+extern void extract_guile (FILE *fp, const char *real_filename,
+                           const char *logical_filename,
+                           flag_context_list_table_ty *flag_table,
+                           msgdomain_list_ty *mdlp);
+
 
 /* Handling of options specific to this language.  */
 
