@@ -1,6 +1,5 @@
-/* List of exported symbols of libgettextsrc on Cygwin.
-   Copyright (C) 2006-2007, 2009-2011, 2013-2015, 2019 Free Software Foundation,
-   Inc.
+/* List of exported symbols of libgettextsrc on Cygwin and native Windows.
+   Copyright (C) 2006-2024 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2006.
 
    This program is free software: you can redistribute it and/or modify
@@ -28,6 +27,7 @@ VARIABLE(format_language_pretty)
 VARIABLE(formatstring_awk)
 VARIABLE(formatstring_boost)
 VARIABLE(formatstring_c)
+VARIABLE(formatstring_cplusplus_brace)
 VARIABLE(formatstring_csharp)
 VARIABLE(formatstring_elisp)
 VARIABLE(formatstring_gcc_internal)
@@ -57,7 +57,6 @@ VARIABLE(formatstring_smalltalk)
 VARIABLE(formatstring_tcl)
 VARIABLE(formatstring_ycp)
 VARIABLE(gram_max_allowed_errors)
-VARIABLE(gram_pos)
 VARIABLE(input_format_po)
 VARIABLE(input_format_properties)
 VARIABLE(input_format_stringtable)
@@ -68,22 +67,14 @@ VARIABLE(omit_header)
 VARIABLE(output_format_po)
 VARIABLE(output_format_properties)
 VARIABLE(output_format_stringtable)
-VARIABLE(pass_obsolete_entries)
 VARIABLE(plural_table)
 VARIABLE(plural_table_size)
 VARIABLE(po_charset_ascii)
 VARIABLE(po_charset_utf8)
 VARIABLE(po_error)
 VARIABLE(po_error_at_line)
-VARIABLE(po_gram_lval)
-VARIABLE(po_lex_charset)
-#if HAVE_ICONV
-VARIABLE(po_lex_iconv)
-#endif
-VARIABLE(po_lex_weird_cjk)
 VARIABLE(po_multiline_error)
 VARIABLE(po_multiline_warning)
-VARIABLE(po_xerror)
-VARIABLE(po_xerror2)
 VARIABLE(syntax_check_name)
+VARIABLE(textmode_xerror_handler_struct)
 VARIABLE(use_first)
